<?php

/**
 *
 * This class is getting information about the current Open iT instalation
 *
 */
class OpeniT_Server_Info {

	private $openitServerConfig;
	private $freedisk;
	private $diskusage;
	private $inodesusage;

	public function __construct() {
		$this->openitServerConfig = new OpeniT_Server_Config();
		$this->getDiskInfo();
	}


	/**
	 * Returns the current installed Open iT version number
	 * @return String Open iT Server Version Number
	 */
	public function getServerVersion() {
		$accserver = "";
		if (!strstr(PHP_OS, "WIN")){
			$accserver = $this->openitServerConfig->get('ROOT_TEMP_DIR') . '/inst/package/accserver.info';
		}else{
			$accserver = $this->openitServerConfig->get('ROOT_DIR') . '/Configuration/version';
		}

		if (file_exists($accserver)) {

			$parser = new Parser(file_get_contents($accserver), ParserSettings::create(ParserSettings::SPLIT, ParserSettings::SPLIT_TYPE_WHITE_SPACE));
				
			while ($parser->hasMore()) {

				// Get line
				$line = $parser->next();

				if (strstr(PHP_OS, "WIN")){
					return $line->get(0)->toString();
				}

				// Insert key, value into array
				if ($line->get(0)->toString() == "VERSION") {
					return $line->get(1)->toString();
				}
			}
		}
	}


	/**
	 * Returning array of packages that is installed
	 * @return mixed Array of strings, packages installed
	 */
	public function getInstalledPackages() {
		$folder = $this->openitServerConfig->get('ROOT_TEMP_DIR') . '/inst/package/';
		$files = glob($folder . '*.info');

		$packages = array();
		foreach ($files as $file) {
			$file = str_replace(array($folder,'.info'), '', $file);
			$packages[] = $file;
		}

		return $packages;
	}


	/**
	 * Returns mixed array of OpeniT_Patch instanses
	 * @return mixed Array of OpeniT_Patch
	 */
	public function getInstalledPatches() {
		$folder = "";
		if (!strstr(PHP_OS, "WIN")){
			$folder = $this->openitServerConfig->get('ROOT_TEMP_DIR') . '/inst/patches/';
		}else{
			$folder = trim($this->openitServerConfig->get('ROOT_DIR')) . '/patches/';
		}

		$pacthes = array();
		$files = null;
		
		if (strstr(PHP_OS, "WIN")){
			$files = glob($folder . 'Patch*');
			foreach ($files as $file) {
				if(is_dir($file)){
					$file = substr($file,strlen($folder)+5);
					$revision = 1;
					$id = str_replace($folder, '', $file);
				}	
				// If we get the revision number, then we do a split
				if (strstr($id, '-')) {
					list ($id, $revision) = explode('-', $id);
				}
				//Making the info null since the patches in Windows doesn't have the information in the patches folder.
				$pacthes[$id] = new OpeniT_Patch($id, null, $revision);
			}
		}else{
			$files = glob($folder . '*');

			foreach ($files as $file) {
				if(stristr($file,"patch.log") || is_dir($file)) continue;
				$revision = 1;
				$id = str_replace($folder, '', $file);
					
				//If the patch is for client don't show it in the dashboard
				if( strpos(file_get_contents($folder . $id),"AFFECTS: Client") === false){
				
					// If we get the revision number, then we do a split
					if (strstr($id, '-')) {
						list ($id, $revision) = explode('-', $id);
					}
					
					$pacthes[$id] = new OpeniT_Patch($id, file_get_contents($file), $revision);
				}
			}
		}
		return $pacthes;
	}

	/**
	 * Get the free disk space, percentage of disk space and inodes used
	 */
	private function getDiskInfo() {

		// OS Checking
		// If windows
		if (strstr(PHP_OS, "WIN")){
			$openitPath = explode(":", $this->openitServerConfig->get('ROOT_DATA_DIR'));
			$openitPath = $openitPath[0] . ':';

			$output = `fsutil volume diskfree $openitPath`;
			$output = explode("\n", $output);
			
			//Get the total disk space
			$totaldisk = explode(" : ",$output[1]);
			$totaldisk = $totaldisk[1];
			
			//Get the free disk space
			$freedisk = explode(" : ",$output[2]);			
			$freedisk = $freedisk[1];
			$this->freedisk = $freedisk / 1024;
			
			//Get the disk usage base on the free disk and total disk space and append 
			$this->diskusage = intval((($totaldisk - $freedisk) / $totaldisk)*100) . "%";
		} else {
			//if linux
			// Find df binary
			$binDF = $this->openitServerConfig->get('BIN_DIR') . '/df';

			// Database path
			$databasePath = $this->openitServerConfig->get('ROOT_DATA_DIR');

			// Execute DF for disk space, and clean up the results
			$diskusage = shell_exec($binDF . ' -k -P -T ' . $databasePath);
			$diskusage = explode("\n", $diskusage);
			$diskusage = preg_replace('/\s\s+/', ' ', $diskusage[1]);

			//Divide the results
			list($fs, $fstype, $fstotal, $fsused, $fsavailable, $fsusage, $fsmount) = explode(" ", $diskusage);
			$this->freedisk = $fsavailable;
			$this->diskusage = $fsusage;


			// Execute DF for inodes, and clean up the results
			$inodesusage = shell_exec($binDF . ' -i -P -T ' . $databasePath);
			$inodesusage = explode("\n", $inodesusage);
			$inodesusage = preg_replace('/\s\s+/', ' ', $inodesusage[1]);

			//Divide the results
			list($fs, $fstype, $intotal, $inused, $inavailable, $inpct, $fsmount) = explode(" ", $inodesusage);
			$this->inodesusage = $inpct;
		}
	}
	
	/**
	 * Returning kb of space left on the database disk
	 * @return int Space of disk free under database, in kb
	 */
	public function getAvailableDatabaseDisk() {

		return $this->freedisk;
	}

	/**
	 * Returning percentage of disk space used
	 * @return string Percentage of disk space used
	 */
	public function getDiskUsage() {

		return $this->diskusage;
	}

	/**
	 * Returning percentage of inodes used
	 * @return int Percentage inodes used
	 */
	public function getInodesUsage() {

		return $this->inodesusage;
	}

	public function getClientLog() {
		// Find where the binary of watchlog is located
		$binWatchLog = $this->openitServerConfig->get('BIN_DIR') . '/watchlog';

		// Run watchlog with nomail and verbose mode, make sure that we add 2>&1, so we get ALL ouput
		$watchLogData = shell_exec($binWatchLog . ' --nomail --verbose 2>&1');

		// Lets pares the watchlog data
		$parser = new Parser($watchLogData, ParserSettings::create(ParserSettings::ONLY_LINES_THAT_STARTS_WITH, 'client='));

		$clients = array();

		while ($parser->hasMore()) {
				
			// Get line
			$line = $parser->next(ParserSettings::create(ParserSettings::SPLIT, ParserSettings::SPLIT_TYPE_WHITE_SPACE));
				
			// Spec from the VERBOSE output, after the first WHITE_SPACE split...
			// client=192.168.210.38
			// type=FL_TY_LICPOL2
			// (daily)
			// now=0,
			// last=0
				
			// Since a parserline returns a new parserline, we can just insert new parsesettings, to split every
			// variable after the spec above..
			$host = $line->get(0, ParserSettings::create(ParserSettings::SPLIT, ParserSettings::SPLIT_TYPE_EQUALS))->get(1)->toString();
			$type = $line->get(1, ParserSettings::create(ParserSettings::SPLIT, ParserSettings::SPLIT_TYPE_EQUALS))->get(1)->toString();
			$freq = ucfirst(str_replace(array('(', ')'), '', $line->get(2)->toString()));
			$now = str_replace(',', '', $line->get(3, ParserSettings::create(ParserSettings::SPLIT, ParserSettings::SPLIT_TYPE_EQUALS))->get(1)->toString());
			$last = $line->get(4, ParserSettings::create(ParserSettings::SPLIT, ParserSettings::SPLIT_TYPE_EQUALS))->get(1)->toString();
				
			// Get client
			if (isset($clients[$host])) {
				$client =& $clients[$host];
			} else {
				$client = null;
				$client = new OpeniT_Client_Transfer_Info($host);
				$clients[$host] =& $client;
			}
				
			$client->add($type, $freq, $now, $last);
				
			unset($client);
				
		}
			
		return $clients;
	}
}

?>